
// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf     : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf   : World < string UIWidget="None"; >;
float4x4 ViewIXf   : ViewInverse < string UIWidget="None"; >;

// UN-TWEAKABLES - Time Parameter AUTOMATICALLY-TRACKED
float Timer : TIME < string UIWidget = "None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName = "Lamp 0 Position";
    string Space  = "World";
> = {-0.5f,2.0f,1.25f};

float3 Lamp0Color : Specular <
    string UIName   = "Lamp 0";
    string Object   = "Pointlight0";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f};

float maxDisplacement <
    string UIWidget = "slider";
    float UIMin = 0.0;
    float UIMax = 1.0;
    float UIStep = 0.05;
    string UIName =  "Maximum Displacement";
> = 0.3;

float frequency <
    string UIWidget = "slider";
    float UIMin = 1;
    float UIMax = 100.0;
    float UIStep = 1;
    string UIName =  "Frequency";
> = 10;

float phaseU <
    string UIWidget = "slider";
    float UIMin = 1;
    float UIMax = 100.0;
    float UIStep = 1;
    string UIName =  "Phase U";
> = 20;

float phaseV <
    string UIWidget = "slider";
    float UIMin = 1;
    float UIMax = 100.0;
    float UIStep = 1;
    string UIName =  "Phase V";
> = 20;


//////// INPUT VERTEX FROM THE STREAM ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 Normal	: NORMAL;
	float2 UV       : TEXCOORD0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float3 LightVec	    : TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
	
    vertexOutput OUT  = (vertexOutput)0;
	
	// transform normal with transposed inverse of world matrix
    OUT.WorldNormal   = mul(IN.Normal,WorldITXf).xyz;
	
	// calculate Phase from time, frequency and phase shifts
	float fPhase   = frequency*Timer + phaseU*IN.UV.y+ phaseV*IN.UV.x;
	
	// displace the vertex along the normal vector
	float3 AnimPos = IN.Position + maxDisplacement*sin(fPhase) * IN.Normal;
	float4 PosModel = float4(AnimPos,1);
	
	// transform to world to calculate light vector
    float3 PosWorld = mul(PosModel,WorldXf).xyz;
    OUT.LightVec = (Lamp0Pos - PosWorld);
	
	// transform to screen space
	OUT.HPosition = mul(PosModel,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 std_PS(vertexOutput IN) : COLOR {
	
    float3 Ln = normalize(IN.LightVec);
    float3 Nn = normalize(IN.WorldNormal);
	
    float ldn = saturate(dot(Ln,Nn));
	
    float3 diffuse = (ldn * Lamp0Color);
	
    return float4(diffuse,1);
}

///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 ProcAnim <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, std_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

// --- end of file ---